# Reference point generation for decomposition based approach (coded by Monalisa Pal)
#################################################################
# Look-up table for standard values of partitions (Standard MOPs)
# n_obj p1  p2  n_pts
# 3     12  0   91
# 5     6   0   210
# 8     3   2   156
# 10    3   2   275
# 15    2   1   135
#################################################################
#  Look-up table for standard values of partitions (MMMOPs)
# n_obj p1  p2  n_pts
# 3     23  0   300
# 5     8   0   495
# 8     5   2   828
# 10    4   3   935
#################################################################


import copy
import numpy as np


class form_ref_pts(object):
    def __init__(self, m, divisions):
        self.M = m - 1
        self.div = divisions
        self.points = []

    def recursive(self, arr, d, l):
        arr_c = copy.deepcopy(arr)
        if d == self.M - 1:
            self.points.append(arr_c)
        else:
            for i in range(0, l):
                node_val = float(i) / float(self.div)
                arr_next = copy.deepcopy(arr_c)
                arr_next.append(node_val)
                self.recursive(arr_next, d + 1, l - i)

    def form(self):
        layer = []
        for i in range(0, self.div + 1):
            layer.append(float(i) / float(self.div))
        for i in range(0, len(layer)):
            l1 = []
            l1.append(layer[i])
            self.recursive(l1, 0, len(layer) - i)
        for i in range(0, len(self.points)):
            s = sum(self.points[i])
            self.points[i].append(1.0 - s)
        self.points = np.asarray(self.points)


def form_refs(dim, outer, inner):
    points = []

    factory = form_ref_pts(dim, outer)
    factory.form()
    factory2 = form_ref_pts(dim, inner)
    factory2.form()
    factory2.points = (factory2.points / 2) + (1.0 / (2.0 * dim))

    for i in range(0, len(factory.points)):
        points.append(factory.points[i])
    for i in range(0, len(factory2.points)):
        points.append(factory2.points[i])

    return np.asarray(points)

n_obj = input('Enter number of objectives:')
n_obj = int(n_obj)
if n_obj < 7:
    print('For ' + str(n_obj) + '-objectives, two-layers are not required.')
    p1 = input('Enter number of divisions:')
    a1 = form_ref_pts(n_obj, int(p1))
    a1.form()
    ref_pts = a1.points
else:
    print('For ' + str(n_obj) + '-objectives, two-layers are required.')
    p1 = input('Enter number of divisions in the boundary layer:')
    p2 = input('Enter number of divisions in the inside layer:')
    ref_pts = form_refs(n_obj, int(p1), int(p2))

output_filename = 'Obj' + str(n_obj) + '.txt'
f = open(output_filename, 'w')
for i in range(len(ref_pts)):
    for j in range(len(ref_pts[0])):
        f.write(repr(ref_pts[i][j]))
        f.write('\t')
    f.write('\n')
f.close()

print(str(len(ref_pts)) + " reference points are written to 'Obj" + str(n_obj) + ".txt'.")
